from netbox.api.viewsets import NetBoxModelViewSet

from .. import  models
from .serializers import VpnConnectionPhase2Serializer, VpnConnectionSerializer


class VpnConnectionViewSet(NetBoxModelViewSet):
    queryset = models.VpnConnection.objects.prefetch_related('tags').annotate(
    )
    serializer_class = VpnConnectionSerializer

class VpnConnectionPhase2ViewSet(NetBoxModelViewSet):
    queryset = models.VpnConnectionPhase2.objects.prefetch_related('tags').annotate(
    )
    serializer_class = VpnConnectionPhase2Serializer


from netbox.api.routers import NetBoxRouter
from . import views


app_name = 'netbox_vpn_plugin'

router = NetBoxRouter()
router.register('connection', views.VpnConnectionViewSet)
router.register('phase2', views.VpnConnectionPhase2ViewSet)

urlpatterns = router.urls



from django.urls import path

from netbox.views.generic import ObjectChangeLogView
from . import models, views

urlpatterns = (

    # Connections
    path('connection/', views.VpnConnectionListView.as_view(), name='vpnconnection_list'),
    path('connection/add/', views.VpnConnectionEditView.as_view(), name='connection_add'),
    path('connection/<int:pk>/', views.VpnConnectionView.as_view(), name='connection'),
    path('connection/<int:pk>/edit/', views.VpnConnectionEditView.as_view(), name='vpnconnection_edit'),
    path('connection/<int:pk>/delete/', views.VpnConnectionDeleteView.as_view(), name='vpnconnection_delete'),
    path('connection/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='vpnconnection_changelog', kwargs={
        'model': models.VpnConnection
    }),

    #phase 2
    path('phase2/', views.VpnConnectionPhase2ListView.as_view(), name='vpnconnectionphase2_list'),
    path('phase2/add/', views.VpnConnectionPhase2EditView.as_view(), name='vpnconnectionphase2_add'),
    path('phase2/<int:pk>/', views.VpnConnectionPhase2View.as_view(), name='phase2'),
    path('phase2/<int:pk>/edit/', views.VpnConnectionPhase2EditView.as_view(), name='vpnconnectionphase2_edit'),
    path('phase2/<int:pk>/delete/', views.VpnConnectionPhase2DeleteView.as_view(), name='vpnconnectionphase2_delete'),
    path('phase2/<int:pk>/changelog/', ObjectChangeLogView.as_view(), name='vpnconnectionphase2_changelog', kwargs={
        'model': models.VpnConnectionPhase2
    }),
)


from extras.plugins import PluginMenuButton, PluginMenuItem
from utilities.choices import ButtonColorChoices


menu_items = (
    PluginMenuItem(
        link='plugins:netbox_vpn_plugin:vpnconnection_list',
        link_text='Connections',
        buttons=[
            PluginMenuButton(
            link='plugins:netbox_vpn_plugin:connection_add',
            title='Add',
            icon_class='mdi mdi-plus-thick',
            color=ButtonColorChoices.GREEN,
            )
        ]        
    ),
)


from extras.plugins import PluginConfig

class NetBoxVpnConfig(PluginConfig):
    name = 'netbox_vpn_plugin'
    verbose_name = 'VPN'
    author = "Henrik Hansen"
    author_email = "henrik.hansen@cgi.com"
    description = 'Manage VPN Configurations for Tenants'
    version = '0.8'
    base_url = 'vpn'
    
config = NetBoxVpnConfig



from django.db.models import Count
from extras.plugins import PluginTemplateExtension
from netbox.views import generic
from netbox_vpn_plugin.filtersets import VpnConnectionFilterSet
from . import  forms, models, tables

class VpnConnectionView(generic.ObjectView):
    queryset = models.VpnConnection.objects.all()
    def get_extra_context(self, request, instance):
        table = tables.VpnConnectionPhase2Table (instance.phase2.all())
        table.configure(request)

        return {
            'phase2_table': table,
        }

class VpnConnectionListView(generic.ObjectListView):
    queryset = models.VpnConnection.objects.all()
    filterset = VpnConnectionFilterSet
    filterset_form = forms.VpnConnectionFilterForm

    action_buttons = ('add',)  
    table = tables.VpnConnectionListTable

class VpnConnectionEditView(generic.ObjectEditView):
    queryset = models.VpnConnection.objects.all()
    form = forms.VpnConnectionForm


class VpnConnectionDeleteView(generic.ObjectDeleteView):
    queryset = models.VpnConnection.objects.all()


# PHase 2

class VpnConnectionPhase2View(generic.ObjectView):
     
    queryset = models.VpnConnectionPhase2.objects.all()
    table = tables.VpnConnectionPhase2Table

class VpnConnectionPhase2ListView(generic.ObjectListView):
    action_buttons = ('add',)  
    queryset = models.VpnConnectionPhase2.objects.all()
    table = tables.VpnConnectionListTable


class VpnConnectionPhase2EditView(generic.ObjectEditView):
    queryset = models.VpnConnectionPhase2.objects.all()
    form = forms.VpnConnectionPhase2Form


class VpnConnectionPhase2DeleteView(generic.ObjectDeleteView):
    queryset = models.VpnConnectionPhase2.objects.all()


from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from ..models import VpnConnection, VpnConnectionPhase2


class VpnConnectionSerializer(NetBoxModelSerializer):
    class Meta:
        model = VpnConnection
        fields = (
            'id', 'display', 
            'vpn_endpoint', 'customer_contact', 'comments', 'change_id', 'ordered_by', 'status', 'active_since','tenant','vpn_endpoint','remote_gateway_ip','nat_traversal','dead_pear_detection','auth_method','pre_shared_key','ike_version','accept_types','accept_peer_id','encrytion','authentication','dh_group','key_lifetime','local_id',
            'created', 'last_updated',
        )

class VpnConnectionPhase2Serializer(NetBoxModelSerializer):
    class Meta:
        model = VpnConnectionPhase2
        fields = (
            'id', 'display', 
            'vpn_connection', 'name', 'local_address', 'remote_address', 'encrytion', 'authentication', 'dh_group', 'key_lifetime', 
            'created', 'last_updated',
        )


# Generated by Django 4.0.5 on 2022-06-24 09:13

import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion
import taggit.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('extras', '0073_journalentry_tags_custom_fields'),
        ('ipam', '0057_created_datetimefield'),
        ('tenancy', '0007_contact_link'),
        ('dcim', '0153_created_datetimefield'),
    ]

    operations = [
        migrations.CreateModel(
            name='VpnConnection',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('remote_organisation', models.CharField(max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('change_id', models.CharField(blank=True, max_length=100, null=True)),
                ('ordered_by', models.CharField(blank=True, max_length=100, null=True)),
                ('status', models.CharField(default='active', max_length=50)),
                ('active_since', models.DateField(blank=True, null=True)),
                ('remote_gateway_ip', models.GenericIPAddressField(blank=True, null=True)),
                ('nat_traversal', models.CharField(default='disable', max_length=50)),
                ('dead_pear_detection', models.CharField(default='disable', max_length=50)),
                ('auth_method', models.CharField(default='pre-shared key', max_length=50)),
                ('pre_shared_key', models.CharField(blank=True, max_length=100, null=True)),
                ('ike_version', models.CharField(default='V2', max_length=50)),
                ('accept_types', models.CharField(default='any peer id', max_length=50)),
                ('accept_peer_id', models.CharField(blank=True, max_length=100, null=True)),
                ('encrytion', models.CharField(default='AES256', max_length=50)),
                ('authentication', models.CharField(default='AES256', max_length=50)),
                ('dh_group', models.CharField(blank=True, max_length=50, null=True)),
                ('key_lifetime', models.CharField(default='3600', max_length=100)),
                ('local_id', models.CharField(blank=True, max_length=100, null=True)),
                ('customer_contact', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='tenancy.contact')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('tenant', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='tenancy.tenant')),
                ('vpn_endpoint', models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='dcim.device')),
            ],
            options={
                'ordering': ('remote_organisation',),
            },
        ),
        migrations.CreateModel(
            name='VpnConnectionPhase2',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=django.core.serializers.json.DjangoJSONEncoder)),
                ('name', models.CharField(max_length=50)),
                ('remote_address', models.CharField(max_length=50)),
                ('encrytion', models.CharField(default='AES256', max_length=50)),
                ('authentication', models.CharField(default='AES256', max_length=50)),
                ('dh_group', models.CharField(blank=True, max_length=50, null=True)),
                ('key_lifetime', models.CharField(default='3600', max_length=100)),
                ('local_address', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='ipam.prefix')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vpn_connection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='phase2', to='netbox_vpn_plugin.vpnconnection')),
            ],
            options={
                'ordering': ('vpn_connection', 'name'),
            },
        ),
    ]



from django import forms
from netbox.forms import NetBoxModelForm
from utilities.forms.fields import CommentField, DynamicModelChoiceField
from netbox.forms import NetBoxModelForm, NetBoxModelBulkEditForm, NetBoxModelFilterSetForm

from .models import VpnConnection, VpnConnectionPhase2
from tenancy.models import Tenant, Contact
from dcim.models import Device
from ipam.models import Prefix

from utilities.forms import (
    DynamicModelChoiceField,
    DynamicModelMultipleChoiceField, StaticSelect,
    APISelect, APISelectMultiple, StaticSelectMultiple, TagFilterField
)

class VpnConnectionForm(NetBoxModelForm):
    comments = CommentField()

    tenant = DynamicModelChoiceField(
        queryset=Tenant.objects.all(),
        required=True
    )
    vpn_endpoint = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=True
    )

    customer_contact = DynamicModelChoiceField(
        queryset=Contact.objects.all(),
        required=False
    )

    fieldsets = (
            ('Organizational', 
                ('remote_organisation', 'tenant', 'ordered_by','change_id','customer_contact','status')
            ),
            ('Network', 
                ('vpn_endpoint','remote_gateway_ip','nat_traversal','dead_pear_detection')
            ),
            ('Auth', 
                ('auth_method','pre_shared_key','ike_version','accept_types','accept_peer_id')
            ),
             ('Phase 1 Proposal', 
                ('encrytion','authentication','dh_group','key_lifetime','local_id')
            ), 
            )

    class Meta:
        model = VpnConnection
        exclude = []


class VpnConnectionFilterForm(NetBoxModelFilterSetForm):
    model = VpnConnection
    q = forms.CharField(
        required=False,
        label='Search'
    )
    tenant = DynamicModelChoiceField(
        queryset=Tenant.objects.all(),
        required=False
    )
    
    tag = TagFilterField(model)



class VpnConnectionPhase2Form(NetBoxModelForm):
    vpn_connection = DynamicModelChoiceField(
        queryset=VpnConnection.objects.all()
    )
    local_address = DynamicModelChoiceField(
        queryset=Prefix.objects.all(),
        required=True
    )


    class Meta:
        model = VpnConnectionPhase2
        fields = (
            'vpn_connection', 'name', 'local_address', 'remote_address', 'encrytion', 'authentication',
            'dh_group', 'key_lifetime', 'tags',
        )




from setuptools import find_packages, setup

setup(
    name='netbox-vpn-plugin',
    version='0.8.1',
    download_url='https://github.com/hhansen06/netbox-vpn-plugin/archive/refs/tags/0.8.1.tar.gz',
    description='Manage VPN Connections in Netbox',
    install_requires=[],
    packages=find_packages(),
    include_package_data=True,
)



from .models import VpnConnection
import django_filters
from extras.filters import TagFilter


class VpnConnectionFilterSet(django_filters.FilterSet):
    q = django_filters.CharFilter(
        method='search',
        label='Search',
    )
    tag = TagFilter()

    class Meta:
        model = VpnConnection
        fields = ['id', 'remote_organisation', 'tenant', 'vpn_endpoint']


    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
                Q(id__icontains=value)
                | Q(number__icontains=value)
                | Q(description__icontains=value)
        )
        return queryset.filter(qs_filter)



import django_tables2 as tables
from netbox.tables import NetBoxTable

from .models import VpnConnection, VpnConnectionPhase2

class VpnConnectionListTable(NetBoxTable):
    remote_organisation = tables.Column(
        linkify=True
    )
    
    class Meta(NetBoxTable.Meta):
        model = VpnConnection
        extra_controls = ('add')
        fields = (
            'pk','id',
            'remote_gateway_ip','vpn_endpoint','remote_organisation','tenant','status',
            'actions'
        )


class VpnConnectionPhase2Table(NetBoxTable):
    vpn_connection = tables.Column(
        linkify=True
    )

    class Meta(NetBoxTable.Meta):
        empty_text = "No Phase 2 Configuration for this vpn defined"
        model = VpnConnectionPhase2
        fields = ('pk','id',
        'name','local_address','remote_address')
        default_columns = ('id','name','local_address','remote_address')



from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.urls import reverse
from pkg_resources import require
from utilities.choices import ChoiceSet

from netbox.models import NetBoxModel

class VpnStatusChoices(ChoiceSet):

    STATUS_OFFLINE = 'offline'
    STATUS_ACTIVE = 'active'
    STATUS_PLANNED = 'planned'
    STATUS_FAILED = 'failed'

    CHOICES = (
        (STATUS_OFFLINE, 'Offline', 'orange'),
        (STATUS_ACTIVE, 'Active', 'green'),
        (STATUS_PLANNED, 'Planned', 'cyan'),
        (STATUS_FAILED, 'Failed', 'red'),
    )

class DeadPeerDetectionChoices(ChoiceSet):

    DISABLE = 'disable'
    ON_IDLE = 'on idle'
    ON_DEMAND = 'on demand'

    CHOICES = (
        (DISABLE, 'disable', 'red'),
        (ON_IDLE, 'on idle', 'green'),
        (ON_DEMAND, 'on demand', 'cyan'),
    )

class NatTraversalChoices(ChoiceSet):

    ENABLE = 'enable'
    DISABLE = 'disable'
    FORCED = 'forced'

    CHOICES = (
        (ENABLE, 'enable', 'red'),
        (DISABLE, 'disable', 'green'),
        (FORCED, 'forced', 'cyan'),
    )

class AuthmethodChoices(ChoiceSet):

    PSK = 'pre-shared key'
    CERT = 'certificate'

    CHOICES = (
        (PSK, 'pre-shared key', 'red'),
        (CERT, 'certificate', 'green'),
    )


class IKEVersionChoices(ChoiceSet):

    IKE_VERSION_1 = 'V1'
    IKE_VERSION_2 = 'V2' 

    CHOICES = [
        (IKE_VERSION_1, 'V1', 'red'),
        (IKE_VERSION_2, 'V2', 'green'),
    ]

class AcceptTypesChoices(ChoiceSet):
    ANY_PEER = 'any peer id'
    SPECIFIC_PEER = 'specific peer id' 

    CHOICES = [
        (ANY_PEER, 'any peer id', 'red'),
        (SPECIFIC_PEER, 'specific peer id', 'green'),
    ]

class EncryptionChoices(ChoiceSet):
    E_DES = 'DES'
    E_3DES = '3DES' 
    E_AES128 = 'AES128' 
    E_AES192 = 'AES192' 
    E_AES256 = 'AES256' 

    CHOICES = [
        (E_DES,'DES','green'),
        (E_3DES,'3DES' ,'green'),
        (E_AES128,'AES128' ,'green'),
        (E_AES192,'AES192' ,'green'),
        (E_AES256,'AES256' ,'green'),
    ]
class DhGroupChoices(ChoiceSet):
    dh_1 = "1"
    dh_2 = "2"
    dh_5 = "5"
    dh_14 = "14"
    dh_15 = "15"
    dh_16 = "16"
    dh_17 = "17"
    dh_18 = "18"
    dh_19 = "19"
    dh_20 = "20"
    dh_21 = "21"
    dh_27 = "27"
    dh_28 = "28"
    dh_29 = "29"
    dh_30 = "30"

    CHOICES = [
        (dh_1,'1','green'),
        (dh_2,'2','green'),
        (dh_5,'5','green'),
        (dh_14,'14','green'),
        (dh_15,'15','green'),
        (dh_16,'16','green'),
        (dh_17,'17','green'),
        (dh_18,'18','green'),
        (dh_19,'19','green'),
        (dh_20,'20','green'),
        (dh_21,'21','green'),
        (dh_27,'27','green'),
        (dh_28,'28','green'),
        (dh_29,'29','green'),
        (dh_30,'30','green'),
    ]

class VpnConnection(NetBoxModel):
    remote_organisation = models.CharField(
        max_length=100,
        help_text="Name of remote Organisation",
    )


    customer_contact = models.ForeignKey(
        to='tenancy.Contact',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True
    )

    comments = models.TextField(
        blank=True,
        help_text=""
    )

    change_id = models.CharField(
        blank=True,
        max_length=100,
        null=True,
        help_text="Related change"
    )

    ordered_by = models.CharField(
        blank=True,
        max_length=100,
        null=True,
        help_text="Member who ordered the tunnel"
    )

    status = models.CharField(
        max_length=50,
        choices=VpnStatusChoices,
        default=VpnStatusChoices.STATUS_ACTIVE
    )

    active_since = models.DateField(
        blank=True,
        null=True
    )

    tenant = models.ForeignKey(
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='+',
        blank=False,
        null=False
    )

    vpn_endpoint = models.ForeignKey(
        to='dcim.Device',
        on_delete=models.PROTECT,
        related_name='+',
        blank=False,
        null=True
    )

    remote_gateway_ip = models.GenericIPAddressField(
       blank=True,
       null=True,
    )
    nat_traversal = models.CharField(
        max_length=50,
        choices=NatTraversalChoices,
        default=NatTraversalChoices.DISABLE
    )
    
    dead_pear_detection = models.CharField(
        max_length=50,
        choices=DeadPeerDetectionChoices,
        default=DeadPeerDetectionChoices.DISABLE
    )


    auth_method = models.CharField(
        max_length=50,
        choices=AuthmethodChoices,
        default=AuthmethodChoices.PSK
    )
    pre_shared_key = models.CharField(
        max_length=100,
        blank=True,
        null=True,
    )
    ike_version = models.CharField(
        max_length=50,
        choices=IKEVersionChoices,
        default=IKEVersionChoices.IKE_VERSION_2
    )
    accept_types = models.CharField(
        max_length=50,
        choices=AcceptTypesChoices,
        default=AcceptTypesChoices.ANY_PEER
    )
    accept_peer_id = models.CharField(
        max_length=100,
        help_text="If Accept types = specific peer ID, define remote peer Id here",
        blank=True,
        null=True,
    )


    encrytion = models.CharField(
        max_length=50,
        choices=EncryptionChoices,
        default=EncryptionChoices.E_AES256
    )
    authentication = models.CharField(
        max_length=50,
        choices=EncryptionChoices,
        default=EncryptionChoices.E_AES256
    )
    dh_group = models.CharField(
        max_length=50,
        choices=DhGroupChoices,
        blank=True,
        null=True,
    )
    key_lifetime = models.CharField(
        max_length=100,
        help_text="in Seconds",
        default= "3600"
    )
    local_id = models.CharField(
        max_length=100,
        blank=True,
        null=True,
    )

    class Meta:
        ordering = ('remote_organisation',)

    def __str__(self):
        return self.remote_organisation

    def get_absolute_url(self):
        return reverse('plugins:netbox_vpn_plugin:connection', args=[self.pk])


class VpnConnectionPhase2(NetBoxModel):
    vpn_connection = models.ForeignKey(
        to=VpnConnection,
        on_delete=models.CASCADE,
        related_name='phase2'
    )
    name = models.CharField(
        max_length=50
    )
    local_address = models.ForeignKey(
        to='ipam.Prefix',
        on_delete=models.PROTECT,
        related_name='+',
    )
    remote_address = models.CharField(
       max_length=50,
       blank=False,
       null=False,
    )
    encrytion = models.CharField(
        max_length=50,
        choices=EncryptionChoices,
        default=EncryptionChoices.E_AES256
    )
    authentication = models.CharField(
        max_length=50,
        choices=EncryptionChoices,
        default=EncryptionChoices.E_AES256
    )
    dh_group = models.CharField(
        max_length=50,
        choices=DhGroupChoices,
        blank=True,
        null=True,
    )    
    key_lifetime = models.CharField(
        max_length=100,
        help_text="in Seconds",
        default= "3600"
    )
    
    class Meta:
        ordering = ('vpn_connection', 'name')

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('plugins:netbox_vpn_plugin:phase2', args=[self.pk])